import 'dart:convert';
import 'dart:io';

import 'package:fas_ui_flutter/components/app_info.dart';
import 'package:fas_ui_flutter/components/indicator_view.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

//选中取消监听
typedef OnSelectedListener = void Function(AppInfo appInfo);

///显示已安装的应用程序
// ignore: must_be_immutable
class AppListDialog extends StatefulWidget {
  OnSelectedListener onSelectedListener;

  AppListDialog({Key key, this.onSelectedListener}) : super(key: key);

  @override
  _AppListDialogState createState() => _AppListDialogState();
}

class _AppListDialogState extends State<AppListDialog> {
  OnSelectedListener _onSelectedListener;
  List<AppInfo> appInfoList = [];
  AppInfo _selectAppInfo;
  PageController _pageController =
      PageController(viewportFraction: 1.0, initialPage: 0, keepPage: true);

  //每页显示的app数量
  int _pageAppCount = 20;
  int _selectPageIndex = 0;

  @override
  void dispose() {
    _pageController.dispose();
    super.dispose();
  }

  Future<Null> _getAppList() async {
    try {
      if (Platform.isAndroid) {
        const platform = const MethodChannel("led_display.flutter.io/app_list");
        final List<dynamic> result = await platform.invokeMethod("getAppList");
        var list = result.cast<String>().toList();
        //{"icon":2131558400,"isRom":true,"isUser":true,"name":"AnyShow","packageName":"com.gti.anyshow"}
        list.forEach((element) {
          //将字符串解析成AppInfo对象
          var appInfo = AppInfo.fromJson(json.decode(element));
          appInfoList.add(appInfo);
        });
        setState(() {});
      }
    } on PlatformException catch (e) {
      print(e.message);
    }
  }

  @override
  void initState() {
    super.initState();
    _onSelectedListener = widget.onSelectedListener;
    _getAppList();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      type: MaterialType.transparency,
      child: Center(
        child: Card(
          margin: EdgeInsets.only(left: 8, right: 8),
          clipBehavior: Clip.antiAlias,
          color: Color(0xFFEEEEEE),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.all(Radius.circular(13)),
          ),
          child: Container(
            height: 626,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _buildTitle(),
                _buildAppList(context),
                _buildButtonViews(context)
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildAppList(BuildContext context) {
    int pageCount = (appInfoList.length / _pageAppCount).ceil();
    return Expanded(
        child: Stack(
      children: [
        Container(
          child: MediaQuery.removePadding(
            removeTop: true,
            removeBottom: true,
            context: context,
            child: PageView.builder(
              reverse: false,
              controller: _pageController,
              onPageChanged: (index) {
                _selectPageIndex = index;
                // setState(() {});
              },
              physics: BouncingScrollPhysics(),
              itemBuilder: (BuildContext context, int index) {
                int startIndex = index * _pageAppCount;
                int endIndex = startIndex + _pageAppCount;
                if (endIndex < appInfoList.length) {
                  return _buildGridView(
                      appInfoList.sublist(startIndex, endIndex));
                } else {
                  return _buildGridView(appInfoList.sublist(startIndex));
                }
              },
              itemCount: pageCount,
            ),
          ),
        ),
        Align(
          alignment: Alignment.bottomCenter,
          child: Container(
              margin: EdgeInsets.only(bottom: 20),
              child: IndicatorView(
                count: pageCount,
                selectIndex: _selectPageIndex,
                pageController: _pageController,
              )),
        )
      ],
    ));
  }

  Widget _buildGridView(List<AppInfo> appInfos) {
    int crossAxisCount = 4;
    double space = (MediaQuery.of(context).size.width - 240) / crossAxisCount;
    return Container(
      margin: EdgeInsets.only(left: 20, right: 20),
      child: GridView.builder(
          itemCount: appInfos.length,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: crossAxisCount,
              mainAxisSpacing: 20,
              mainAxisExtent: 80,
              crossAxisSpacing: space),
          itemBuilder: (context, index) {
            return _buildItem(index, appInfos[index]);
          }),
    );
  }

  Widget _buildTitle() {
    return Container(
      margin: EdgeInsets.only(left: 12, top: 13, bottom: 20),
      child: Text("全部APP", style: TextStyle(fontSize: 13, color: Colors.black)),
    );
  }

  Widget _buildItem(int index, AppInfo appInfo) {
    return InkWell(
      onTap: () {
        appInfoList.forEach((element) {
          if (appInfo != element) element.isChecked = false;
        });
        appInfo.isChecked = !appInfo.isChecked;
        if (appInfo.isChecked) {
          _selectAppInfo = appInfo;
        } else {
          _selectAppInfo = null;
        }
        setState(() {});
      },
      child: Stack(
        children: [
          Container(
            height: 80,
            width: 60,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Image.file(
                  File(appInfo.icon),
                  height: 60,
                  width: 60,
                  fit: BoxFit.cover,
                ),
                Expanded(
                  child: Container(
                    alignment: Alignment.bottomCenter,
                    child: Text(
                      appInfo.name,
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(
                        fontSize: 12,
                        color: Colors.black,
                      ),
                    ),
                  ),
                )
              ],
            ),
          ),
          Positioned(
            child: appInfo.isChecked
                ? Icon(Icons.check_circle, color: Color(0xFF697FFE), size: 15)
                : Icon(Icons.circle, color: Color(0xFF606060), size: 15),
            top: 6,
            right: 6,
          )
        ],
      ),
    );
  }

  ///两个按钮部分
  Widget _buildButtonViews(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
          border: Border(top: BorderSide(color: Colors.black54, width: 0.34))),
      height: 44,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: [
          Expanded(
            child: InkWell(
              onTap: () {
                Navigator.of(context).pop();
              },
              child: Container(
                  decoration: BoxDecoration(
                      border: Border(
                          right:
                              BorderSide(color: Colors.black54, width: 0.17))),
                  alignment: Alignment.center,
                  child: Text("取消")),
            ),
          ),
          Expanded(
              child: InkWell(
            onTap: () {
              if (!_selectAppInfo.isChecked) {
                //如果点击确定按钮的时候没有选中任何一个数据，则提示未选中
                ScaffoldMessenger.of(context).showSnackBar(SnackBar(
                  content: Text("未选中应用程序"),
                  behavior: SnackBarBehavior.floating,
                ));
                return;
              }

              if (_onSelectedListener != null && _selectAppInfo.isChecked) {
                _onSelectedListener(_selectAppInfo);
                Navigator.of(context).pop();
              }
            },
            child: Container(
                decoration: BoxDecoration(
                    border: Border(
                        left: BorderSide(color: Colors.black54, width: 0.17))),
                alignment: Alignment.center,
                child: Text("确定")),
          )),
        ],
      ),
    );
  }
}
